"""
@author syt123450 / https://github.com/syt123450
"""

import sys

sys.path.insert(0, "./src/tf")

from saved_model import preprocess_saved_model
from frozen_model import preprocess_frozen_model
from hdf5_combined_model import preprocess_hdf5_combined_model
from hdf5_separated_model import preprocess_hdf5_separated_model


def show_tf_model_summary(path_model):
    print(path_model)
    print("tensorflow Model Summary...")


def preprocess_tensorflow_model(input_format, path_model, path_output_dir, output_node_names=None):
    if input_format == "tf_saved_model":
        preprocess_saved_model(
            path_model,
            path_output_dir,
            output_node_names
        )
    elif input_format == "tf_frozen_model":
        preprocess_frozen_model(
            path_model,
            path_output_dir,
            output_node_names
        )
    elif input_format == "tf_hdf5_model":
        preprocess_hdf5_combined_model(
            path_model,
            path_output_dir,
            output_node_names
        )
    elif input_format == "tf_hdf5_separated_model":
        preprocess_hdf5_separated_model(
            path_model,
            path_output_dir,
            output_node_names
        )
    else:
        print("Preprocess nothing for tensorflow model.")

    print("Mission Complete!!!")



from utility.file_utility import valid_file, valid_directory, show_invalid_message

TS_NODE_BIN_PATH = './src/tf/pb2json/node_modules/.bin/ts-node'
CONVERTER_SCRIPT_PATH = './src/tf/pb2json/tools/pb2json_converter.ts'


def convert(path_input, path_output):
    print('pb to json')
    import subprocess
    subprocess.check_call(["node", TS_NODE_BIN_PATH, CONVERTER_SCRIPT_PATH, path_input, path_output])



"""
@author syt123450 / https://github.com/syt123450
"""

import os
import shutil
from tf.pb2json.pb2json_conversion import convert
import subprocess

input_format_config = '--input_format=tf_saved_model'


def preprocess_saved_model(input_path, output_path, output_node_names):
    print("preprocessing tensorflow saved model...")
    if not os.path.isdir:
        os.mkdir(output_path + '/tmp')
    subprocess.check_call([
        "tensorflowjs_converter",
        input_format_config,
        "--output_node_names=" + output_node_names,
        "--saved_model_tags=serve",
        input_path,
        output_path + '/tmp'
    ])
    path_now = os.getcwd()
    os.chdir(output_path)
    absolute_output_path = os.getcwd()
    absolute_output_path_temp = absolute_output_path + '/tmp/'
    os.chdir(path_now)
    convert(
        absolute_output_path_temp,
        absolute_output_path
    )
    shutil.rmtree(absolute_output_path_temp)



from utility.file_utility import valid_file, valid_directory, show_invalid_message


def process_tfjs_model(path_input, path_output, output_names=None):
    import subprocess
    if (not valid_file(path_input)):
        show_invalid_message('input model file', path_input)
        return
    if (not valid_directory(path_output)):
        show_invalid_message('output directory', path_output)
        return

    MAIN_JS_PATH = "./src/tfjs/main.js"
    if output_names is None:
        subprocess.check_call(["node", MAIN_JS_PATH, path_input, path_output])
    else:
        output_names = "--output_layer_names=" + output_names
        subprocess.check_call(["node", MAIN_JS_PATH, output_names, path_input, path_output])


def show_tfjs_model_summary(path_input):
    import subprocess
    if not valid_file(path_input):
        show_invalid_message('input model file', path_input)
        return

    MAIN_JS_PATH = "./src/tfjs/main.js"
    subprocess.check_call(["node", MAIN_JS_PATH, "--summary", path_input])



"""
@author syt123450 / https://github.com/syt123450
"""

import os
import subprocess
import tensorflow as tf


temp_model_name = '/enc_model.h5'


def preprocess_hdf5_separated_model(input_path, output_path, output_node_names):
    print("Preprocess hdf5 separated model.")
    model = load_separate_model(input_path)

    temp_enc_model_path = output_path + temp_model_name
    enc_model = generate_encapsulate_model(model, output_node_names)
    save(enc_model, temp_enc_model_path)
    convert_h5_to_tfjs(temp_enc_model_path, output_path)
    remove_temp_enc_model(temp_enc_model_path)


def load_separate_model(input_path):
    print("Load model into memory...")
    model_paths = input_path.split(",")
    with open(model_paths[0], 'r') as f:
        model = tf.keras.models.model_from_json(f.read())
    model.load_weights(model_paths[1])
    return model


def generate_encapsulate_model(model, output_layer_names):
    print("Generate multi-output encapsulated model...")
    layer_name_list = output_layer_names.split(",")
    display_model = tf.keras.models.Model(
        inputs=model.input,
        outputs=list(map(lambda oln: model.get_layer(oln).output, layer_name_list))
    )
    return display_model


def save(enc_model, output_path):
    print("Save temp multi-output .h5 model...")
    enc_model.compile(optimizer='adam',
                      loss='sparse_categorical_crossentropy',
                      metrics=['accuracy'])
    tf.keras.models.save_model(
        enc_model,
        output_path,
        overwrite=True,
        include_optimizer=True
    )


def convert_h5_to_tfjs(model_path, output_path):
    print("Convert .h5 to web friendly format...")
    subprocess.check_call([
        "tensorflowjs_converter",
        "--input_format=keras",
        model_path,
        output_path
    ])


def remove_temp_enc_model(model_path):
    print("Delete temp .h5 model...")
    os.remove(model_path)
